package com.gic.finance.service.outer.impl;

import java.util.Date;

import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.AsyncCallbackUtils;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.constant.InvoiceInitiateTypeConstants;
import com.gic.finance.constant.InvoiceStatusEnum;
import com.gic.finance.dto.InvoiceManageDTO;
import com.gic.finance.dto.OperationUserInfoDTO;
import com.gic.finance.dto.callback.InvoiceCallbackDTO;
import com.gic.finance.entity.TabInvoiceAccount;
import com.gic.finance.entity.TabInvoiceManage;
import com.gic.finance.qo.InvoiceManageListQueryQO;
import com.gic.finance.service.InvoiceAccountService;
import com.gic.finance.service.InvoiceManageApiService;
import com.gic.finance.service.InvoiceManageService;

@Service("invoiceManageApiService")
public class InvoiceManageApiServiceImpl implements InvoiceManageApiService {
    private static final Logger LOGGER = LogManager.getLogger(InvoiceManageApiServiceImpl.class);
    @Autowired
    private InvoiceManageService invoiceManageService;
    @Autowired
    private InvoiceAccountService invoiceAccountService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private UserApiService userApiService;

    public ServiceResponse<String> applyInvoiceInit(InvoiceManageDTO invoiceManageDTO) {
        ServiceResponse paramResponse = ValidParamsUtils.allCheckValidate(invoiceManageDTO,
                InvoiceManageDTO.ApplyInvoiceValid.class);
        if (!paramResponse.isSuccess()) {
            return paramResponse;
        }
        if (invoiceManageDTO.getBillingAmount() < 1000) {
            //如果开票金额小于1000，则不允许开票
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "开票金额小于1000");
        }
        invoiceManageDTO.setCreateTime(new Date());
        invoiceManageDTO.setUpdateTime(new Date());
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.TO_BE_ISSUED.getCode());
        invoiceManageDTO.setInvoiceApplySerial(CreateRandomUtils.createSerialNumber());
        if (StringUtils.isBlank(invoiceManageDTO.getEnterpriseName())) {
            invoiceManageDTO.setEnterpriseName("暂无商户名称");
        }
        if (StringUtils.isBlank(invoiceManageDTO.getCompanyName())) {
            invoiceManageDTO.setCompanyName("暂无公司名称");
        }
        if (StringUtils.isBlank(invoiceManageDTO.getLogo())) {
            invoiceManageDTO.setLogo("暂无公司logo");
        }

        invoiceManageService.save(invoiceManageDTO);
        return ServiceResponse.success(invoiceManageDTO.getInvoiceApplySerial());
    }

    @Override
    public ServiceResponse<String> applyInvoice(InvoiceManageDTO invoiceManageDTO) {
        //计费中心
        invoiceManageDTO.setInitiateType(InvoiceInitiateTypeConstants.BILLING);
        return applyInvoiceInit(invoiceManageDTO);
    }

    @Override
    public ServiceResponse<String> applyInvoiceForApp(InvoiceManageDTO invoiceManageDTO) {
        if (StringUtils.isBlank(invoiceManageDTO.getInitiator())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发起人(initiator)主键不能为空");
        }
        //应用市场类型
        invoiceManageDTO.setInitiateType(InvoiceInitiateTypeConstants.APP);
        return applyInvoiceInit(invoiceManageDTO);
    }

    @Override
    public ServiceResponse<Void> invoice(Integer invoiceManageId, String expressMailName, String expressMailNumber, OperationUserInfoDTO dto) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        if (StringUtils.isBlank(expressMailName) || StringUtils.isBlank(expressMailNumber)) {
            //待邮寄
            invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.TO_BE_MAILED.getCode());
            tabInvoiceManage.setInvoiceStatus(InvoiceStatusEnum.TO_BE_MAILED.getCode());
        } else {
            invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.MAILED.getCode());
            //物流信息
            invoiceManageDTO.setExpressMailName(expressMailName);
            invoiceManageDTO.setExpressMailNumber(expressMailNumber);

            tabInvoiceManage.setInvoiceStatus(InvoiceStatusEnum.MAILED.getCode());
            tabInvoiceManage.setExpressMailName(expressMailName);
            tabInvoiceManage.setExpressMailNumber(expressMailNumber);
        }
        getOperationUserInfo(invoiceManageDTO, dto);

        //开票户信息插入
        TabInvoiceAccount account = invoiceAccountService.getEnable();
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "没有启用的开票户信息，请启用");
        }
        invoiceManageDTO.setInvoicerId(account.getInvoiceAccountId());
        invoiceManageService.update(invoiceManageDTO);

        //回调
        if (tabInvoiceManage.getInitiateType().intValue() == InvoiceInitiateTypeConstants.APP) {
            InvoiceCallbackDTO callbackDTO = EntityUtil.changeEntityNew(InvoiceCallbackDTO.class, tabInvoiceManage);
            AsyncCallbackUtils.callBack(callbackDTO, "com.gic.open.api.service.market.InvoiceCallbackApiService",
                    "invoiceCallback");
        }

        sendSms(tabInvoiceManage, "GICJFZX010");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rejectInvoice(Integer invoiceManageId, String rejectReason, OperationUserInfoDTO dto) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.REJECT.getCode());
        if (StringUtils.isBlank(rejectReason)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "驳回理由不能为空");
        }
        invoiceManageDTO.setRejectReason(rejectReason);
        getOperationUserInfo(invoiceManageDTO, dto);

        invoiceManageService.update(invoiceManageDTO);

        //回调
        if (tabInvoiceManage.getInitiateType().intValue() == InvoiceInitiateTypeConstants.APP) {
            tabInvoiceManage.setInvoiceStatus(InvoiceStatusEnum.REJECT.getCode());
            tabInvoiceManage.setRejectReason(rejectReason);
            InvoiceCallbackDTO callbackDTO = EntityUtil.changeEntityNew(InvoiceCallbackDTO.class, tabInvoiceManage);
            AsyncCallbackUtils.callBack(callbackDTO, "com.gic.open.api.service.market.InvoiceCallbackApiService",
                    "invoiceCallback");
        }

        sendSms(tabInvoiceManage, "GICJFZX011");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> cancelInvoice(Integer invoiceManageId) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.CANCEL.getCode());
        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success(tabInvoiceManage.getInvoiceApplySerial());
    }

    @Override
    public ServiceResponse<Page<InvoiceManageDTO>> listInvoiceManage(InvoiceManageListQueryQO params) {
        com.github.pagehelper.Page page = invoiceManageService.listInvoiceManage(params);
        Page<InvoiceManageDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, InvoiceManageDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<InvoiceManageDTO> getById(Integer invoiceManageId) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceManageDTO.class, tabInvoiceManage));
    }

    @Override
    public ServiceResponse<Void> cancelInvoiceForApp(String initiator) {
        LOGGER.info("应用市场取消开票：{}", initiator);
        TabInvoiceManage record = invoiceManageService.selectInvoiceByInitiator(initiator,
                InvoiceInitiateTypeConstants.APP);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，查无数据");
        }
        if (record.getInvoiceStatus().intValue() != InvoiceStatusEnum.TO_BE_ISSUED.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "不是" + InvoiceStatusEnum.TO_BE_ISSUED.getMessage() + "状态，无法取消");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(record.getInvoiceManageId());
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.CANCEL.getCode());
        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success();
    }

    private void getOperationUserInfo(InvoiceManageDTO dto, OperationUserInfoDTO userInfo) {
        dto.setOperatorId(userInfo.getId().toString());
        dto.setOperatorName(userInfo.getRealName());
        dto.setOperatorPhone(userInfo.getUserMobile());
        dto.setOperatorTime(new Date());
    }

    private void sendSms(TabInvoiceManage tabInvoiceManage, String smsSceneId) {
        if (StringUtils.isNotBlank(tabInvoiceManage.getExpressMailName())) {
            //如果是计费中心，则发起人数据 就是用户ID
            if (StringUtils.isNotBlank(tabInvoiceManage.getInitiator())) {
                Integer userId = Integer.parseInt(tabInvoiceManage.getInitiator());
                ServiceResponse<UserDTO> userResponse = userApiService.getUserById(userId);
                if (!userResponse.isSuccess()) {
                    LOGGER.warn("发票审批完成发送短信:{},{}", userResponse.getCode(), userResponse.getMessage());
                }
                UserDTO userDTO = userResponse.getResult();

                String[] smsArr = null;
                if ("GICJFZX010".equals(smsSceneId)) {
                    smsArr=  new String[]{tabInvoiceManage.getEnterpriseName(),
                            tabInvoiceManage.getInvoiceApplySerial(),
                            tabInvoiceManage.getExpressMailName(),
                            tabInvoiceManage.getExpressMailNumber()};
                } else if ("GICJFZX011".equals(smsSceneId)) {
                    smsArr = new String[]{tabInvoiceManage.getEnterpriseName(),
                            tabInvoiceManage.getInvoiceApplySerial(), tabInvoiceManage.getRejectReason()};
                }

                ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(smsSceneId,
                        tabInvoiceManage.getEnterpriseId(),
                        userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                if (!smsSendResult.isSuccess()) {
                    LOGGER.warn(smsSendResult.getMessage());
                }
            }
        }
    }
}
